// Usare l'integrato FT2232H - Versione 0.2 - Dicembre 2009
// Copyright  2009, VincenzoV.net (http://www.vincenzov.net)
// Creative Commons | Attribuzione-Condividi allo stesso modo 3.0 Unported.
// Creative Commons | Attribution-Share Alike 3.0 Unported
// http://www.vincenzov.net/tutorial/elettronica-di-base/FT2232H/index.htm

// Please, read AN 109 from FTDI site

#include "stdafx.h"
#include <windows.h>
#include "..\FTCI2C.h"

#define CLOCK_DIVISOR 79 // Set I2C bus clock frequency. Equivalent to 375KHz (if divisor set to 5) 
#define I2C_DATA 0x23    // Data to write to EEPROM (0-255, 0x0-0xFF)
#define I2C_ADDR 0x101   // Address (0-0x7FF)

int _tmain(int argc, _TCHAR* argv[])
{DWORD DevicesFound;          // Number of Ftxxx found
 FTC_STATUS ErrorCode;        // Error code (FTC_SUCCESS mean no errors)
 char DeviceNameBuffer[100];  // Device name string from I2C_GetHiSpeedDeviceNameLocIDChannel()
 DWORD LocationID;            // Where is located FTxxx (undused from user)
 char Channel[5];             // A or B (undused)
 DWORD HiSpeedDeviceType;     // FT2232H or FT4232H(undused)
 FTC_HANDLE IicDeviceHandle;  // Handle og opened device
 DWORD clock;                 // I2C bus clock (375 kHz, about 400 kHz max frequency)
 WriteControlByteBuffer WriteControlBuffer;
                              // Command to send to I2C devices
 WriteDataByteBuffer WriteDataBuffer;
                              // Data to send to I2C devices
 FTC_PAGE_WRITE_DATA PageWriteData; 
                              // Unused!
 ErrorCode = I2C_GetNumHiSpeedDevices(&DevicesFound); // Get the number of available FTx232H 
                                                      // NB: FT2232H appares as 2 devices...
 if (ErrorCode != FTC_SUCCESS)                        // Any error?
      {printf ("Error! %d", ErrorCode ); return -1;}
 else printf ("Found %d devices\n", DevicesFound );  

 // Get some identifiers of device 0. Change first parameter accordig your system
 ErrorCode = I2C_GetHiSpeedDeviceNameLocIDChannel(0, DeviceNameBuffer, 100, &LocationID, Channel, 5, &HiSpeedDeviceType);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -2;}
 else  printf ("Using device 0: %s\n", DeviceNameBuffer);
 
 // Open device and inizialize IicDeviceHandle as device handle
 ErrorCode = I2C_OpenHiSpeedDevice(DeviceNameBuffer, LocationID, Channel, &IicDeviceHandle);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -3;}
 else  printf ("Device opened\n");

 // Set FT2232H software compatible with legacy FTxxx - Non required but change CLOCK_DIVISOR meaning
 I2C_TurnOnDivideByFiveClockingHiSpeedDevice(IicDeviceHandle);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -8;}
 else  printf ("Invalid handle...\n");
 
 // Inizialize device as I2C master
 ErrorCode = I2C_InitDevice(IicDeviceHandle, CLOCK_DIVISOR);
 if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode ); return -5;}
 else 
      {I2C_GetHiSpeedDeviceClock(CLOCK_DIVISOR, &clock);  // Verify clock - Non required
      printf ("Device initialized; clock = %d Hz\n", clock);}

// SetUp address (See 24LC16 Data sheet - "Device addressing, Byte write")
// I2C_ADDR (11 bit address):  A10 A9 A8 A7 A6 A5 A4 A3 A2 A1 A0
// I2C_ADDR >> 7: A10 A9 A8 A7
// (I2C_ADDR >> 7) & 0x0E: A10 A9 A8 0
// ((I2C_ADDR >> 7) & 0x0E) | 0xA0: 1 0 1 0 A10 A9 A8 0
WriteControlBuffer[0] = ((I2C_ADDR >> 7) & 0x0E) | 0xA0; 
WriteControlBuffer[1] = I2C_ADDR & 0xFF; // A7 A6 A5 A4 A3 A2 A1 A0

WriteDataBuffer[0] = I2C_DATA;  // Data to write (single byte)

// Write DATA e COMMAND to I2C devices
ErrorCode = I2C_Write(IicDeviceHandle, &WriteControlBuffer, 2, true, 20, true, BYTE_WRITE_TYPE, &WriteDataBuffer, 1, true, 20, &PageWriteData);
if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode ); return -6;}
else  printf ("Write data %d to %d\n",I2C_DATA, I2C_ADDR);

// Read back DATA
WriteDataBuffer[0] = 0;  // Clear data... is not a trick

ErrorCode = I2C_Read(IicDeviceHandle, &WriteControlBuffer, 2, true, 20, BYTE_READ_TYPE, &WriteDataBuffer, 1);
if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode ); return -6;}
else  printf ("Read %d form %d\n",WriteDataBuffer[0], I2C_ADDR);

// Close device
ErrorCode = I2C_Close(IicDeviceHandle);
if (ErrorCode != FTC_SUCCESS)
      {printf ("Error! %d", ErrorCode );return -4;}
else  printf ("Device closed\n");

return 0;
}
