// Usare l'integrato FT2232H - Versione 0.2 - Dicembre 2009
// Copyright  2009, VincenzoV.net (http://www.vincenzov.net)
// Creative Commons | Attribuzione-Condividi allo stesso modo 3.0 Unported.
// Creative Commons | Attribution-Share Alike 3.0 Unported
// http://www.vincenzov.net/tutorial/elettronica-di-base/FT2232H/index.htm

// Please, read AN 109 from FTDI site

#include "stdafx.h" 
#include <windows.h> 
#include "..\FTCI2C.h"   // Change according your system configuration

#define CLOCK_DIVISOR 79 // Set I2C bus clock frequency. Equivalent to 375KHz (if divisor set to 5) 
#define DAC_DATA 0x80    // Data to write to DAC, eight bit (0x80 set Vout to 1/2 Vref, about 2.5V) 
#define DAC_ADDR 0x5E    // 0101 1110 A0=A1=A2=1 - See MAX520 data sheet: Address Byte
#define DAC_CTRL 0x00    // 0000 0x00 - Use DAC0 - See MAX520 data sheet: Command Byte

int _tmain(int argc, _TCHAR* argv[])
{DWORD DevicesFound;          // Number of Ftxxx found
 FTC_STATUS ErrorCode;        // Error code (FTC_SUCCESS mean no errors)
 char DeviceNameBuffer[100];  // Device name string from I2C_GetHiSpeedDeviceNameLocIDChannel()
 DWORD LocationID;            // Where is located FTxxx (undused from user)
 char Channel[5];             // A or B (undused)
 DWORD HiSpeedDeviceType;     // FT2232H or FT4232H(undused)
 FTC_HANDLE IicDeviceHandle;  // Handle og opened device
 DWORD clock;                 // I2C bus clock (375 kHz, about 400 kHz max frequency)
 WriteControlByteBuffer WriteControlBuffer;
                              // Command to send to I2C devices
 WriteDataByteBuffer WriteDataBuffer;
                              // Data to send to I2C devices
 FTC_PAGE_WRITE_DATA PageWriteData; 
                              // Unused!

 ErrorCode = I2C_GetNumHiSpeedDevices(&DevicesFound); // Get the number of available FTx232H 
                                                      // NB: FT2232H appares as 2 devices...
 if (ErrorCode != FTC_SUCCESS)                        // Any error?
      {printf ("Error! %d", ErrorCode ); return -1;}
 else printf ("Found %d devices\n", DevicesFound );  

 // Get some identifiers of device 0. Change first parameter accordig your system
 ErrorCode = I2C_GetHiSpeedDeviceNameLocIDChannel(0, DeviceNameBuffer, 100, &LocationID, Channel, 5, &HiSpeedDeviceType);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -2;}
 else  printf ("Using device 0: %s\n", DeviceNameBuffer);
 
 // Open device and inizialize IicDeviceHandle as device handle
 ErrorCode = I2C_OpenHiSpeedDevice(DeviceNameBuffer, LocationID, Channel, &IicDeviceHandle);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -3;}
 else  printf ("Device opened\n");

 // Set FT2232H software compatible with legacy FTxxx - Non required but change CLOCK_DIVISOR meaning
 I2C_TurnOnDivideByFiveClockingHiSpeedDevice(IicDeviceHandle);
 if (ErrorCode != FTC_SUCCESS)
       {printf ("Error! %d", ErrorCode ); return -8;}
 else  printf ("Invalid handle...\n");
 
 // Inizialize device as I2C master
 ErrorCode = I2C_InitDevice(IicDeviceHandle, CLOCK_DIVISOR);
 if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode ); return -5;}
 else 
      {I2C_GetHiSpeedDeviceClock(CLOCK_DIVISOR, &clock);  // Verify clock - Non required
      printf ("Device initialized; clock = %d Hz\n", clock);}

WriteControlBuffer[0] = DAC_ADDR;  // See MAX520 data sheet: Address Byte
WriteControlBuffer[1] = DAC_CTRL;  // See MAX520 data sheet: Command Byte

WriteDataBuffer[0] = DAC_DATA;    // Data to write - 8 bit 0-255

// Write DATA e COMMAND to I2C devices
ErrorCode = I2C_Write(IicDeviceHandle, &WriteControlBuffer, 2, true, 20, true, BYTE_WRITE_TYPE, &WriteDataBuffer, 1, true, 20, &PageWriteData);
if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode ); return -6;}
else  printf ("Write data %d \n",DAC_DATA);

// CLose device
ErrorCode = I2C_Close(IicDeviceHandle);
if (ErrorCode != FTC_SUCCESS)
      {printf ("Erroe! %d", ErrorCode );return -4;}
else  printf ("Device closed\n");

return 0;
}
